// voronoi.cpp
// MCS 481 project 3 description version 1.0
// David Dumas
//
// Compute the Voronoi diagram and print to stdout
// in "vertex-list + edge-pairs" format
//
// Based on CGAL example "Triangulation_2/voronoi.cpp"

#include <iostream>
#include <iterator>
#define _USE_MATH_DEFINES
#include <cmath>
#include <map>
#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
#include <CGAL/Delaunay_triangulation_2.h>

typedef CGAL::Exact_predicates_inexact_constructions_kernel K;
typedef CGAL::Delaunay_triangulation_2<K>  DT;

int main(int argc, char **argv)
{
  // Construct the voronoi diagram, reading sites from cin/stdin
  std::istream_iterator<DT::Point> begin(std::cin);
  std::istream_iterator<DT::Point> end;

  std::map<DT::Point,int,K::Less_xy_2> finite_indices;
  std::map<K::Direction_2,int> infinite_indices;

  DT dt;
  dt.insert(begin, end);

  int n=0;
  // Print the finite vertices and assign indices
  for (DT::Face_iterator fit = dt.faces_begin(); fit != dt.faces_end(); ++fit) {
    DT::Point p = dt.dual(fit);
    finite_indices.insert( std::pair<DT::Point,int>(p,n++) );
    std::cout << p << std::endl;
  }

  // Print the infinite vertices and assign indices
  for (DT::Edge_iterator eit = dt.edges_begin(); eit != dt.edges_end(); ++eit) {
    CGAL::Object o = dt.dual(eit);
    if (const K::Ray_2 *r = CGAL::object_cast<K::Ray_2>(&o)) {
      K::Direction_2 dir = r->direction();
      infinite_indices.insert( std::pair<K::Direction_2,int>(dir,n++) );
      double turns = 0.5 * atan2(dir.dy(), dir.dx()) / M_PI;
      std::cout << (turns>=0.0?turns:turns+1.0) << std::endl;
    }
  }

  std::cout << std::endl;

  // Print the edges as pairs of indices
  for (DT::Edge_iterator eit = dt.edges_begin(); eit != dt.edges_end(); ++eit) {    
    CGAL::Object o = dt.dual(eit);
    if (const K::Segment_2 *s = CGAL::object_cast<K::Segment_2>(&o)) {
      std::cout << finite_indices[s->vertex(0)] << " " << finite_indices[s->vertex(1)] << std::endl;
    } else if (const K::Ray_2 *r = CGAL::object_cast<K::Ray_2>(&o)) {
      std::cout << finite_indices[r->source()] << " " << infinite_indices[r->direction()] << std::endl;
    }
  }
  return 0;
}
